# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2001,2002,2003 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# $Id: _envtoxml.awk,v 1.1 2004/08/08 21:15:38 carlo Exp $
# =====================================================================
# int _envtoxml([string template, string fileroot, string eval,\
#		string table, boolean trim, string tago, string tagc,\
#		string ofile, int lvl, boolean parse])
#
# Take environment variables and use them to replace special tags in
# an XML template file. All arguments are optional, but at least the
# 'template' parameter should be passed, otherwise the result will
# hardly be of any usefulness :-)
# =====================================================================

function _envtoxml(tpl,froot,eval,tbl,trim,tago,tagc,ofile,lvl,parse,   \
	ltago,ltagc,retago,retagc,nl,end,tlen,tag,enc,a,i,\
	input,record,j,start) {

  i=j=0

  # Check args and set defaults.
  if (tpl == "") tpl = "/dev/null"
  if (ofile == "") ofile = _STDOUT
  if (froot == "") froot = ENVIRON["CSA_ROOT"] "/xml/"
  if (tago == "") tago = "$["
  if (tagc == "") tagc = "]"
  if (tago == tagc) return _EINVAL # start- and end-tags must be different
  if (!ENVIRON["CSA_TPLMAX"]) ENVIRON["CSA_TPLMAX"] = 10

  # prevent infinite recursions.
  if (lvl > ENVIRON["CSA_TPLMAX"]) return 1

  ltago = length(tago)
  ltagc = length(tagc)

  # Prepare the substitution regexp's.
  retago = _escreg(tago)
  retagc = _escreg(tagc)

  retago = retago "[-A-Za-z0-9_/.:]+" retagc

  if (!lvl) {				# only if 1st recursion.

     # Load an extra NoSQL table if requested. This data is also loaded
     # in ENVIRON[], so it can be used to override data in there. Table
     # rows will be assigned to ENVIRON["_TBL1"], ENVIRON["_TBL2"], etc.,
     # with ENVIRON["_TBL0"] being the No. of records in table. Any table
     # header information is ignored. The names used (one lower-case
     # letter follwed by one or more numbers), are CSA-reserved names
     # (see the docs), and the "t" suffix stands for "table". Note that
     # splitting table records into fields and unescaping NoSQL special
     # sequences is up to whoever will make use of these values, often
     # through the user-level hook _outvar().

     if (tbl != "") {
        ENVIRON["_TBL0"] = 0
        while (getline record < tbl > 0) {
	   # Store table header in a separate place.
	   if (record ~ /^\001/) {
	      _TBLH = record
	      continue
	   }
	   # Append row to ENVIRON[].
	   ENVIRON["_TBL" ++ENVIRON["_TBL0"]] = record
        }

        close(tbl)

        _tbltordf()			# prepare also the RDF version.
     }

     for (j in ENVIRON) {

         # Always skip rc(1) functions, confidential data
         # and other CSA-specific stuff.

         if (j ~ /(^fn_|^HTTP_COOKIE|PGPPASS$|^CSA_AUTH_PW|^csa0)/) {
	    ENVIRON[j] = "<!-- removed -->"
	    continue
         }


         # If CSA_AUTH_SSL is in effect, let's be careful
         # about including HTTP_AUTHORIZATION in templates.

         if (j == "HTTP_AUTHORIZATION" && ENVIRON["CSA_AUTH_SSL"]) {
	      if (ENVIRON["SSL_CIPHER_USEKEYSIZE"] == "" && \
					ENVIRON["HTTPS"] != "on") {
	         # This cannot be set to anything other than "",
	         # or there will be problems with envtoxml.awk.
	         ENVIRON[j] = ""
	         continue
	      }
         }

         # Call local output hook on non-CSA environment variables.
         if (j !~ /^[Cc][Ss][Aa]/) ENVIRON[j] = _outvar(j,ENVIRON[j])

         # Include also URI-, URP-, ISO- and XML-PCDATA- encodings in
         # ENVIRON[] by default. I need to prepare all three encodings
         # because this function is used for both HTML and XML templates.
         # Note that if the string is to be included in an XML document
         # and it must represent a URI, then the the URI_ or URP_ versions,
         # whichever applies, should be used. ISO_ encoding is practically
         # only useful with HTML, and it is of no use with XML.

         ENVIRON["URI_" j] = _uriencode(ENVIRON[j])
         ENVIRON["URP_" j] = _uriencode(ENVIRON[j],1)
         ENVIRON["ISO_" j] = _isoencode(ENVIRON[j])
         ENVIRON["XML_" j] = _xmlencode(ENVIRON[j])
     }

     # Tell the local hook that defs are over.
     _outvar("_ENOMSG",_ENOMSG)
  }

  if (eval != "") split(eval,a,",")

  # Process the XML template file.

  while (getline input < tpl > 0) {
     nl = 1
     do {
        match(input,retago)
        if (RSTART) {
	   end = index(input, tagc)
	   tlen = end - RSTART - ltago
	   start = RSTART		# save before calling _rdfprint()
	   _rdfprint(substr(input, 1, start - 1),parse,ofile)
	   tag = substr(input, start + ltago, tlen)

	   # Now handle encoding flags.
	   if (sub(/:c$/,_NULL,tag)) {
	      # character-data, to be included verbatim.
	      enc = "cdata"
	   }
	   else if (sub(/:p$/,_NULL,tag)) {
	      # parsed-character data.
	      enc = "pcdata"
	   }
	   else enc = "base64"			# default is binary.

	   for (i in a) {			# Is "tag" to be eval'ed ?
	       if (a[i] == tag) {
		  tag = ENVIRON[tag]		# Get tag value from env.
		  break
	       }
	   }

	   if (tag ~ /\//) {			# Catch file patterns.
	      if (tag ~ /\.\./) {		# Catch any '..' pattern.
		 _rdfprint("$[" tag "]",parse,ofile)  # Print tag literally.
	      }
	      else {
		 # Read external file, if it exists.
		 if (froot != _NULL && tag !~ /^\//) {
		    tag = froot "/" tag
		    gsub(/\/\//, "/", tag)	# Turn any "//" into "/".
		 }

		 if (enc == "base64") {		# Base64.
		    system("exec csa-encdec -e -b " tag) > ofile
		 }
		 else if (enc == "cdata") {
		    # embeddable character-data, include verbatim.
		    _envtoxml(tag,froot,eval,tbl,trim,tago,tagc,ofile,lvl+1,0)
		 }
		 else {
		    # parsed-character data, whatever "parsed" means in
		    # the specific context (XML in this case).
		    _envtoxml(tag,froot,eval,tbl,trim,tago,tagc,ofile,lvl+1,1)
		 }
		 nl = 0
	      }
	   }
	   else {
	      if (trim) ENVIRON[tag] = _strip(ENVIRON[tag])
	      _rdfprint(ENVIRON[tag],parse,ofile)
	   }

	   input = substr(input, end + ltagc)
	}
	else {
	   _rdfprint(input,parse,ofile)
	   input = _NULL		# Not to loop forever.
	}
     }  while (length(input))

     if (nl) _rdfprint("\n",0,ofile)
  }

  close(tpl)
}

