# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2001,2002,2003 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# $Id: _uriencode.awk,v 1.1 2004/08/08 21:15:38 carlo Exp $
# =====================================================================
# string _uriencode(string s, boolean urp)
#
# Perform URI-encoding of special characters, according to RFC 1378. The
# set of characters that are URI-encoded is:
#
#  % # & ? + SPACE = " \t \f ' \n \r $ ( ) [ ] ` < > \ / . |
#  ; ! * { } : ~ ,
#
# Note how '%' must be escaped first, and SPACE must be done after
# '+'. See http://www.apache.org/info/css-security/ for more info on
# the so-called Cross-Site Scripting vulnerabilities. If the 'urp'
# ragument is set to '1', then the encoding is done twice for some of
# the involved characters. This meant to produce values that are to
# be included in PATH_INFO elements, rather than in the QUERY_STRING,
# as the content of PATH_INFO is unencoded a first time by the CGI
# interface itself (verified with both Apache and Boa), and then by the
# CSA decode() function. If the data contain characters such as / ? and
# &, and they are encoded only once, they will be made available to CSA
# already unencoded, and that would disrupt a number of things (for
# instance, splitting PATH_INFO on slashes would become unreliable, to
# name only one).
# =====================================================================

function _uriencode(string,urp) {

   if (urp) {
      gsub(/&/, "%26", string)
      gsub(/\?/, "%3F", string)
      gsub(/=/, "%3D", string)
      gsub(/\//, "%2F", string)
   }

   # Order matters here.
   gsub(/%/, "%25", string)
   gsub(/\+/, "%2B", string)
   gsub(/  */, "+", string)

   # Order does not matter here.
   gsub(/\t/, "%09", string)
   gsub(/\f/, "%0C", string)
   gsub(/\n/, "%0A", string)
   gsub(/'/, "%27", string)
   gsub(/#/, "%23", string)
   gsub(/&/, "%26", string)
   gsub(/\?/, "%3F", string)
   gsub(/=/, "%3D", string)
   gsub(/"/, "%22", string)
   gsub(/\r/, "%0D", string)
   gsub(/\$/, "%24", string)
   gsub(/\(/, "%28", string)
   gsub(/\)/, "%29", string)
   gsub(/\[/, "%5B", string)
   gsub(/]/, "%5D", string)
   gsub(/`/, "%60", string)
   # gsub(/\001/, "%01", string)
   gsub(/</, "%3C", string)
   gsub(/>/, "%3E", string)
   gsub(/\\/, "%5C", string)
   gsub(/\//, "%2F", string)
   gsub(/\./, "%2E", string)
   gsub(/\|/, "%7C", string)
   gsub(/;/, "%3B", string)
   gsub(/!/, "%21", string)
   gsub(/\*/, "%2A", string)
   gsub(/{/, "%7B", string)
   gsub(/}/, "%7D", string)
   gsub(/:/, "%3A", string)
   gsub(/~/, "%7E", string)
   gsub(/,/, "%2C", string)

   # Add more encodings here, if necessary.

   return string
}

