# =====================================================================
# sendmail.awk: shell front-end to the _sendmail() AWK function.
#
# Copyright (c) 2004 Carlo Strozzi
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# $Id: envtoxml.awk,v 1.2 2004/08/08 21:15:37 carlo Exp $

BEGIN {

  # Check context and arguments, and set defaults.

  if (tpl !~ /^\//) exit(rc=1)				# no default!
  if (tbl !~ /^\//) tbl = "/dev/null"			# full-path ?

  # Load an extra NoSQL table if requested. Table rows will be
  # assigned to _TBL[1], _TBL[2], etc., with _TBL[0] being the No. of
  # records in table. Any table header information is ignored. Note
  # that splitting table records into fields and unescaping NoSQL
  # special sequences is up to whoever will make use of these values,
  # often through the application-level _userproc() function.

  if (tbl != "") {
     _TBL[0] = 0
     while (getline record < tbl > 0) {
	# Store table header in a separate place.
	if (record ~ /^\001/) {
	   _TBLH = record
	   continue
	}
	# Append raw table record to _TBL[].
	_TBL[++_TBL[0]] = record
     }

     close(tbl)
  }

  for (j in ENVIRON) {

      # Always skip rc(1) functions, confidential data
      # and other CSA-specific stuff.

      if (j ~ /(^(fn_|HTTP_COOKIE|CSA_AUTH_PW|csa0)|PGPPASS$)/) {
	 ENVIRON[j] = "<!-- removed -->"
	 continue
      }


      # Remove other security-sensitive stuff.

      if (j == "HTTP_AUTHORIZATION") {
	 ENVIRON[j] = ""
	 continue
      }

      # Only CSA_xxx variables are automatically encoded, while all
      # others are left up to the application-level processor.

      if (j ~ /^CSA_/) _response(j,ENVIRON[j])
  }

  # Call the application-level processor. To prevent _userproc()
  # from being called twice, with unpredictable results, the call to
  # _userproc() is skipped if we are alredy being called as part of
  # write-back statements written by _userproc() to CSA_TPLWB during
  # template response processing.

  if (ENVIRON["CSA_TPLWB"] == _NULL) _userproc(_O_RESPONSE)

  exit(rc=_sendmail(tpl,eval,trim))

}

# EOF
